# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Session History Service

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model


class ApimodelsXRayTicketResult(Model):
    """Apimodels X ray ticket result (apimodels.XRayTicketResult)

    Properties:
        elapsed_time: (elapsedTime) REQUIRED float

        id_: (id) REQUIRED str

        matchpool: (matchpool) REQUIRED str

        pod_name: (podName) REQUIRED str

        reason: (reason) REQUIRED str

        region: (region) REQUIRED str

        tick_id: (tickId) REQUIRED str

        ticket_id: (ticketId) REQUIRED str

        ticket_status: (ticketStatus) REQUIRED str

        timestamp: (timestamp) REQUIRED str

        is_backfill: (isBackfill) OPTIONAL bool

        is_pivot: (isPivot) OPTIONAL bool

        member_attributes: (memberAttributes) OPTIONAL Dict[str, Any]

        total_players: (totalPlayers) OPTIONAL int
    """

    # region fields

    elapsed_time: float  # REQUIRED
    id_: str  # REQUIRED
    matchpool: str  # REQUIRED
    pod_name: str  # REQUIRED
    reason: str  # REQUIRED
    region: str  # REQUIRED
    tick_id: str  # REQUIRED
    ticket_id: str  # REQUIRED
    ticket_status: str  # REQUIRED
    timestamp: str  # REQUIRED
    is_backfill: bool  # OPTIONAL
    is_pivot: bool  # OPTIONAL
    member_attributes: Dict[str, Any]  # OPTIONAL
    total_players: int  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_elapsed_time(self, value: float) -> ApimodelsXRayTicketResult:
        self.elapsed_time = value
        return self

    def with_id(self, value: str) -> ApimodelsXRayTicketResult:
        self.id_ = value
        return self

    def with_matchpool(self, value: str) -> ApimodelsXRayTicketResult:
        self.matchpool = value
        return self

    def with_pod_name(self, value: str) -> ApimodelsXRayTicketResult:
        self.pod_name = value
        return self

    def with_reason(self, value: str) -> ApimodelsXRayTicketResult:
        self.reason = value
        return self

    def with_region(self, value: str) -> ApimodelsXRayTicketResult:
        self.region = value
        return self

    def with_tick_id(self, value: str) -> ApimodelsXRayTicketResult:
        self.tick_id = value
        return self

    def with_ticket_id(self, value: str) -> ApimodelsXRayTicketResult:
        self.ticket_id = value
        return self

    def with_ticket_status(self, value: str) -> ApimodelsXRayTicketResult:
        self.ticket_status = value
        return self

    def with_timestamp(self, value: str) -> ApimodelsXRayTicketResult:
        self.timestamp = value
        return self

    def with_is_backfill(self, value: bool) -> ApimodelsXRayTicketResult:
        self.is_backfill = value
        return self

    def with_is_pivot(self, value: bool) -> ApimodelsXRayTicketResult:
        self.is_pivot = value
        return self

    def with_member_attributes(
        self, value: Dict[str, Any]
    ) -> ApimodelsXRayTicketResult:
        self.member_attributes = value
        return self

    def with_total_players(self, value: int) -> ApimodelsXRayTicketResult:
        self.total_players = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "elapsed_time"):
            result["elapsedTime"] = float(self.elapsed_time)
        elif include_empty:
            result["elapsedTime"] = 0.0
        if hasattr(self, "id_"):
            result["id"] = str(self.id_)
        elif include_empty:
            result["id"] = ""
        if hasattr(self, "matchpool"):
            result["matchpool"] = str(self.matchpool)
        elif include_empty:
            result["matchpool"] = ""
        if hasattr(self, "pod_name"):
            result["podName"] = str(self.pod_name)
        elif include_empty:
            result["podName"] = ""
        if hasattr(self, "reason"):
            result["reason"] = str(self.reason)
        elif include_empty:
            result["reason"] = ""
        if hasattr(self, "region"):
            result["region"] = str(self.region)
        elif include_empty:
            result["region"] = ""
        if hasattr(self, "tick_id"):
            result["tickId"] = str(self.tick_id)
        elif include_empty:
            result["tickId"] = ""
        if hasattr(self, "ticket_id"):
            result["ticketId"] = str(self.ticket_id)
        elif include_empty:
            result["ticketId"] = ""
        if hasattr(self, "ticket_status"):
            result["ticketStatus"] = str(self.ticket_status)
        elif include_empty:
            result["ticketStatus"] = ""
        if hasattr(self, "timestamp"):
            result["timestamp"] = str(self.timestamp)
        elif include_empty:
            result["timestamp"] = ""
        if hasattr(self, "is_backfill"):
            result["isBackfill"] = bool(self.is_backfill)
        elif include_empty:
            result["isBackfill"] = False
        if hasattr(self, "is_pivot"):
            result["isPivot"] = bool(self.is_pivot)
        elif include_empty:
            result["isPivot"] = False
        if hasattr(self, "member_attributes"):
            result["memberAttributes"] = {
                str(k0): v0 for k0, v0 in self.member_attributes.items()
            }
        elif include_empty:
            result["memberAttributes"] = {}
        if hasattr(self, "total_players"):
            result["totalPlayers"] = int(self.total_players)
        elif include_empty:
            result["totalPlayers"] = 0
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        elapsed_time: float,
        id_: str,
        matchpool: str,
        pod_name: str,
        reason: str,
        region: str,
        tick_id: str,
        ticket_id: str,
        ticket_status: str,
        timestamp: str,
        is_backfill: Optional[bool] = None,
        is_pivot: Optional[bool] = None,
        member_attributes: Optional[Dict[str, Any]] = None,
        total_players: Optional[int] = None,
        **kwargs,
    ) -> ApimodelsXRayTicketResult:
        instance = cls()
        instance.elapsed_time = elapsed_time
        instance.id_ = id_
        instance.matchpool = matchpool
        instance.pod_name = pod_name
        instance.reason = reason
        instance.region = region
        instance.tick_id = tick_id
        instance.ticket_id = ticket_id
        instance.ticket_status = ticket_status
        instance.timestamp = timestamp
        if is_backfill is not None:
            instance.is_backfill = is_backfill
        if is_pivot is not None:
            instance.is_pivot = is_pivot
        if member_attributes is not None:
            instance.member_attributes = member_attributes
        if total_players is not None:
            instance.total_players = total_players
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> ApimodelsXRayTicketResult:
        instance = cls()
        if not dict_:
            return instance
        if "elapsedTime" in dict_ and dict_["elapsedTime"] is not None:
            instance.elapsed_time = float(dict_["elapsedTime"])
        elif include_empty:
            instance.elapsed_time = 0.0
        if "id" in dict_ and dict_["id"] is not None:
            instance.id_ = str(dict_["id"])
        elif include_empty:
            instance.id_ = ""
        if "matchpool" in dict_ and dict_["matchpool"] is not None:
            instance.matchpool = str(dict_["matchpool"])
        elif include_empty:
            instance.matchpool = ""
        if "podName" in dict_ and dict_["podName"] is not None:
            instance.pod_name = str(dict_["podName"])
        elif include_empty:
            instance.pod_name = ""
        if "reason" in dict_ and dict_["reason"] is not None:
            instance.reason = str(dict_["reason"])
        elif include_empty:
            instance.reason = ""
        if "region" in dict_ and dict_["region"] is not None:
            instance.region = str(dict_["region"])
        elif include_empty:
            instance.region = ""
        if "tickId" in dict_ and dict_["tickId"] is not None:
            instance.tick_id = str(dict_["tickId"])
        elif include_empty:
            instance.tick_id = ""
        if "ticketId" in dict_ and dict_["ticketId"] is not None:
            instance.ticket_id = str(dict_["ticketId"])
        elif include_empty:
            instance.ticket_id = ""
        if "ticketStatus" in dict_ and dict_["ticketStatus"] is not None:
            instance.ticket_status = str(dict_["ticketStatus"])
        elif include_empty:
            instance.ticket_status = ""
        if "timestamp" in dict_ and dict_["timestamp"] is not None:
            instance.timestamp = str(dict_["timestamp"])
        elif include_empty:
            instance.timestamp = ""
        if "isBackfill" in dict_ and dict_["isBackfill"] is not None:
            instance.is_backfill = bool(dict_["isBackfill"])
        elif include_empty:
            instance.is_backfill = False
        if "isPivot" in dict_ and dict_["isPivot"] is not None:
            instance.is_pivot = bool(dict_["isPivot"])
        elif include_empty:
            instance.is_pivot = False
        if "memberAttributes" in dict_ and dict_["memberAttributes"] is not None:
            instance.member_attributes = {
                str(k0): v0 for k0, v0 in dict_["memberAttributes"].items()
            }
        elif include_empty:
            instance.member_attributes = {}
        if "totalPlayers" in dict_ and dict_["totalPlayers"] is not None:
            instance.total_players = int(dict_["totalPlayers"])
        elif include_empty:
            instance.total_players = 0
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, ApimodelsXRayTicketResult]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[ApimodelsXRayTicketResult]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        ApimodelsXRayTicketResult,
        List[ApimodelsXRayTicketResult],
        Dict[Any, ApimodelsXRayTicketResult],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "elapsedTime": "elapsed_time",
            "id": "id_",
            "matchpool": "matchpool",
            "podName": "pod_name",
            "reason": "reason",
            "region": "region",
            "tickId": "tick_id",
            "ticketId": "ticket_id",
            "ticketStatus": "ticket_status",
            "timestamp": "timestamp",
            "isBackfill": "is_backfill",
            "isPivot": "is_pivot",
            "memberAttributes": "member_attributes",
            "totalPlayers": "total_players",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "elapsedTime": True,
            "id": True,
            "matchpool": True,
            "podName": True,
            "reason": True,
            "region": True,
            "tickId": True,
            "ticketId": True,
            "ticketStatus": True,
            "timestamp": True,
            "isBackfill": False,
            "isPivot": False,
            "memberAttributes": False,
            "totalPlayers": False,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Platform Service

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model

from ..models.fulfill_bundled_item_result import FulfillBundledItemResult


class FulfillItemResult(Model):
    """Fulfill item result (FulfillItemResult)

    Properties:
        error: (error) OPTIONAL str

        item_id: (itemId) OPTIONAL str

        item_sku: (itemSku) OPTIONAL str

        items: (items) OPTIONAL List[FulfillBundledItemResult]
    """

    # region fields

    error: str  # OPTIONAL
    item_id: str  # OPTIONAL
    item_sku: str  # OPTIONAL
    items: List[FulfillBundledItemResult]  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_error(self, value: str) -> FulfillItemResult:
        self.error = value
        return self

    def with_item_id(self, value: str) -> FulfillItemResult:
        self.item_id = value
        return self

    def with_item_sku(self, value: str) -> FulfillItemResult:
        self.item_sku = value
        return self

    def with_items(self, value: List[FulfillBundledItemResult]) -> FulfillItemResult:
        self.items = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "error"):
            result["error"] = str(self.error)
        elif include_empty:
            result["error"] = ""
        if hasattr(self, "item_id"):
            result["itemId"] = str(self.item_id)
        elif include_empty:
            result["itemId"] = ""
        if hasattr(self, "item_sku"):
            result["itemSku"] = str(self.item_sku)
        elif include_empty:
            result["itemSku"] = ""
        if hasattr(self, "items"):
            result["items"] = [
                i0.to_dict(include_empty=include_empty) for i0 in self.items
            ]
        elif include_empty:
            result["items"] = []
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        error: Optional[str] = None,
        item_id: Optional[str] = None,
        item_sku: Optional[str] = None,
        items: Optional[List[FulfillBundledItemResult]] = None,
        **kwargs,
    ) -> FulfillItemResult:
        instance = cls()
        if error is not None:
            instance.error = error
        if item_id is not None:
            instance.item_id = item_id
        if item_sku is not None:
            instance.item_sku = item_sku
        if items is not None:
            instance.items = items
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> FulfillItemResult:
        instance = cls()
        if not dict_:
            return instance
        if "error" in dict_ and dict_["error"] is not None:
            instance.error = str(dict_["error"])
        elif include_empty:
            instance.error = ""
        if "itemId" in dict_ and dict_["itemId"] is not None:
            instance.item_id = str(dict_["itemId"])
        elif include_empty:
            instance.item_id = ""
        if "itemSku" in dict_ and dict_["itemSku"] is not None:
            instance.item_sku = str(dict_["itemSku"])
        elif include_empty:
            instance.item_sku = ""
        if "items" in dict_ and dict_["items"] is not None:
            instance.items = [
                FulfillBundledItemResult.create_from_dict(
                    i0, include_empty=include_empty
                )
                for i0 in dict_["items"]
            ]
        elif include_empty:
            instance.items = []
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, FulfillItemResult]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[FulfillItemResult]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        FulfillItemResult, List[FulfillItemResult], Dict[Any, FulfillItemResult]
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "error": "error",
            "itemId": "item_id",
            "itemSku": "item_sku",
            "items": "items",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "error": False,
            "itemId": False,
            "itemSku": False,
            "items": False,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

"""Auto-generated package that contains models used by the AccelByte Gaming Services Gdpr Service."""

__version__ = "2.15.0"
__author__ = "AccelByte"
__email__ = "dev@accelbyte.net"

# pylint: disable=line-too-long

from .admin_get_platform_acco_c23147 import (
    AdminGetPlatformAccountClosureServicesConfiguration,
)
from .admin_get_services_conf_be46ef import AdminGetServicesConfiguration
from .admin_reset_platform_ac_c4b31b import (
    AdminResetPlatformAccountClosureServicesConfiguration,
)
from .admin_reset_services_co_abf06c import AdminResetServicesConfiguration
from .admin_update_platform_a_1f24aa import (
    AdminUpdatePlatformAccountClosureServicesConfiguration,
)
from .admin_update_services_c_ea010e import AdminUpdateServicesConfiguration
from .delete_admin_email_conf_009cca import DeleteAdminEmailConfiguration
from .get_admin_email_configuration import GetAdminEmailConfiguration
from .save_admin_email_configuration import SaveAdminEmailConfiguration
from .update_admin_email_conf_71e966 import UpdateAdminEmailConfiguration



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

"""Auto-generated package that contains models used by the AccelByte Gaming Services Legal Service."""

__version__ = "4.0.0"
__author__ = "AccelByte"
__email__ = "dev@accelbyte.net"

# pylint: disable=line-too-long

from .create_policy_version import CreatePolicyVersion
from .publish_policy_version import PublishPolicyVersion
from .retrieve_single_policy_version import RetrieveSinglePolicyVersion
from .update_policy_version import UpdatePolicyVersion



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Iam Service

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model

from ..models.accountcommon_client_module_permission import (
    AccountcommonClientModulePermission,
)
from ..models.clientmodel_default_field_value import ClientmodelDefaultFieldValue


class ClientmodelClientTemplate(Model):
    """Clientmodel client template (clientmodel.ClientTemplate)

    Properties:
        basic_required_permissions: (basicRequiredPermissions) REQUIRED List[AccountcommonClientModulePermission]

        default_values: (defaultValues) REQUIRED List[ClientmodelDefaultFieldValue]

        description: (description) REQUIRED str

        id_: (id) REQUIRED str

        required_fields: (requiredFields) REQUIRED List[str]

        type_: (type) REQUIRED str
    """

    # region fields

    basic_required_permissions: List[AccountcommonClientModulePermission]  # REQUIRED
    default_values: List[ClientmodelDefaultFieldValue]  # REQUIRED
    description: str  # REQUIRED
    id_: str  # REQUIRED
    required_fields: List[str]  # REQUIRED
    type_: str  # REQUIRED

    # endregion fields

    # region with_x methods

    def with_basic_required_permissions(
        self, value: List[AccountcommonClientModulePermission]
    ) -> ClientmodelClientTemplate:
        self.basic_required_permissions = value
        return self

    def with_default_values(
        self, value: List[ClientmodelDefaultFieldValue]
    ) -> ClientmodelClientTemplate:
        self.default_values = value
        return self

    def with_description(self, value: str) -> ClientmodelClientTemplate:
        self.description = value
        return self

    def with_id(self, value: str) -> ClientmodelClientTemplate:
        self.id_ = value
        return self

    def with_required_fields(self, value: List[str]) -> ClientmodelClientTemplate:
        self.required_fields = value
        return self

    def with_type(self, value: str) -> ClientmodelClientTemplate:
        self.type_ = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "basic_required_permissions"):
            result["basicRequiredPermissions"] = [
                i0.to_dict(include_empty=include_empty)
                for i0 in self.basic_required_permissions
            ]
        elif include_empty:
            result["basicRequiredPermissions"] = []
        if hasattr(self, "default_values"):
            result["defaultValues"] = [
                i0.to_dict(include_empty=include_empty) for i0 in self.default_values
            ]
        elif include_empty:
            result["defaultValues"] = []
        if hasattr(self, "description"):
            result["description"] = str(self.description)
        elif include_empty:
            result["description"] = ""
        if hasattr(self, "id_"):
            result["id"] = str(self.id_)
        elif include_empty:
            result["id"] = ""
        if hasattr(self, "required_fields"):
            result["requiredFields"] = [str(i0) for i0 in self.required_fields]
        elif include_empty:
            result["requiredFields"] = []
        if hasattr(self, "type_"):
            result["type"] = str(self.type_)
        elif include_empty:
            result["type"] = ""
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        basic_required_permissions: List[AccountcommonClientModulePermission],
        default_values: List[ClientmodelDefaultFieldValue],
        description: str,
        id_: str,
        required_fields: List[str],
        type_: str,
        **kwargs,
    ) -> ClientmodelClientTemplate:
        instance = cls()
        instance.basic_required_permissions = basic_required_permissions
        instance.default_values = default_values
        instance.description = description
        instance.id_ = id_
        instance.required_fields = required_fields
        instance.type_ = type_
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> ClientmodelClientTemplate:
        instance = cls()
        if not dict_:
            return instance
        if (
            "basicRequiredPermissions" in dict_
            and dict_["basicRequiredPermissions"] is not None
        ):
            instance.basic_required_permissions = [
                AccountcommonClientModulePermission.create_from_dict(
                    i0, include_empty=include_empty
                )
                for i0 in dict_["basicRequiredPermissions"]
            ]
        elif include_empty:
            instance.basic_required_permissions = []
        if "defaultValues" in dict_ and dict_["defaultValues"] is not None:
            instance.default_values = [
                ClientmodelDefaultFieldValue.create_from_dict(
                    i0, include_empty=include_empty
                )
                for i0 in dict_["defaultValues"]
            ]
        elif include_empty:
            instance.default_values = []
        if "description" in dict_ and dict_["description"] is not None:
            instance.description = str(dict_["description"])
        elif include_empty:
            instance.description = ""
        if "id" in dict_ and dict_["id"] is not None:
            instance.id_ = str(dict_["id"])
        elif include_empty:
            instance.id_ = ""
        if "requiredFields" in dict_ and dict_["requiredFields"] is not None:
            instance.required_fields = [str(i0) for i0 in dict_["requiredFields"]]
        elif include_empty:
            instance.required_fields = []
        if "type" in dict_ and dict_["type"] is not None:
            instance.type_ = str(dict_["type"])
        elif include_empty:
            instance.type_ = ""
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, ClientmodelClientTemplate]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[ClientmodelClientTemplate]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        ClientmodelClientTemplate,
        List[ClientmodelClientTemplate],
        Dict[Any, ClientmodelClientTemplate],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "basicRequiredPermissions": "basic_required_permissions",
            "defaultValues": "default_values",
            "description": "description",
            "id": "id_",
            "requiredFields": "required_fields",
            "type": "type_",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "basicRequiredPermissions": True,
            "defaultValues": True,
            "description": True,
            "id": True,
            "requiredFields": True,
            "type": True,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import HeaderStr
from ....core import get_namespace as get_services_namespace
from ....core import run_request
from ....core import run_request_async
from ....core import same_doc_as

from ..models import BulkOperationResult
from ..models import CampaignBatchNameChange
from ..models import CampaignBatchNameInfo
from ..models import CampaignCreate
from ..models import CampaignDynamicInfo
from ..models import CampaignInfo
from ..models import CampaignPagingSlicedResult
from ..models import CampaignUpdate
from ..models import CodeCreate
from ..models import CodeCreateResult
from ..models import CodeInfo
from ..models import CodeInfoPagingSlicedResult
from ..models import ErrorEntity
from ..models import RedeemHistoryPagingSlicedResult
from ..models import RedeemRequest
from ..models import RedeemResult
from ..models import ValidationErrorEntity

from ..operations.campaign import ApplyUserRedemption
from ..operations.campaign import BulkDisableCodes
from ..operations.campaign import BulkEnableCodes
from ..operations.campaign import CreateCampaign
from ..operations.campaign import CreateCodes
from ..operations.campaign import DisableCode
from ..operations.campaign import Download
from ..operations.campaign import EnableCode
from ..operations.campaign import GetCampaign
from ..operations.campaign import GetCampaignDynamic
from ..operations.campaign import GetCode
from ..operations.campaign import QueryCampaignBatchNames
from ..operations.campaign import QueryCampaigns
from ..operations.campaign import QueryCodes
from ..operations.campaign import QueryRedeemHistory
from ..operations.campaign import RenameBatch
from ..operations.campaign import UpdateCampaign
from ..models import (
    CampaignCreateRedeemTypeEnum,
    CampaignCreateStatusEnum,
    CampaignCreateTypeEnum,
)
from ..models import (
    CampaignInfoRedeemTypeEnum,
    CampaignInfoStatusEnum,
    CampaignInfoTypeEnum,
)
from ..models import CampaignUpdateRedeemTypeEnum, CampaignUpdateStatusEnum
from ..models import CodeInfoRedeemTypeEnum, CodeInfoStatusEnum, CodeInfoTypeEnum


@same_doc_as(ApplyUserRedemption)
def apply_user_redemption(
    body: RedeemRequest,
    user_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Redeem code (applyUserRedemption)

    [SERVICE COMMUNICATION ONLY] Redeem code. If the campaign which the code belongs to is INACTIVE, the code couldn't be redeemed even if its status is ACTIVE.
    Other detail info:

      * Returns : Redeem result

    Properties:
        url: /platform/admin/namespaces/{namespace}/users/{userId}/redemption

        method: POST

        tags: ["Campaign"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED RedeemRequest in body

        namespace: (namespace) REQUIRED str in path

        user_id: (userId) REQUIRED str in path

    Responses:
        200: OK - RedeemResult (successful operation)

        404: Not Found - ErrorEntity (37142: Code [{code}] does not exist in namespace [{namespace}])

        409: Conflict - ErrorEntity (37172: Campaign [{campaignId}] is inactive in namespace [{namespace}] | 37173: Code [{code}] is inactive in namespace [{namespace}] | 37174: Exceeded max redeem count per code [{maxCount}] | 37175: Exceeded max redeem count per code per user [{maxCount}] | 37177: Code redemption not started | 37178: Code redemption already ended)

        422: Unprocessable Entity - ValidationErrorEntity (20002: validation error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = ApplyUserRedemption.create(
        body=body,
        user_id=user_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(ApplyUserRedemption)
async def apply_user_redemption_async(
    body: RedeemRequest,
    user_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Redeem code (applyUserRedemption)

    [SERVICE COMMUNICATION ONLY] Redeem code. If the campaign which the code belongs to is INACTIVE, the code couldn't be redeemed even if its status is ACTIVE.
    Other detail info:

      * Returns : Redeem result

    Properties:
        url: /platform/admin/namespaces/{namespace}/users/{userId}/redemption

        method: POST

        tags: ["Campaign"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED RedeemRequest in body

        namespace: (namespace) REQUIRED str in path

        user_id: (userId) REQUIRED str in path

    Responses:
        200: OK - RedeemResult (successful operation)

        404: Not Found - ErrorEntity (37142: Code [{code}] does not exist in namespace [{namespace}])

        409: Conflict - ErrorEntity (37172: Campaign [{campaignId}] is inactive in namespace [{namespace}] | 37173: Code [{code}] is inactive in namespace [{namespace}] | 37174: Exceeded max redeem count per code [{maxCount}] | 37175: Exceeded max redeem count per code per user [{maxCount}] | 37177: Code redemption not started | 37178: Code redemption already ended)

        422: Unprocessable Entity - ValidationErrorEntity (20002: validation error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = ApplyUserRedemption.create(
        body=body,
        user_id=user_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(BulkDisableCodes)
def bulk_disable_codes(
    campaign_id: str,
    batch_name: Optional[str] = None,
    batch_no: Optional[List[int]] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Bulk disable codes (bulkDisableCodes)

    Bulk disable codes.

    Bulk disable campaign codes, all matched codes will be disabled except those have already been redeemed.
    Other detail info:

      * Returns : the number of code actually disabled

    Properties:
        url: /platform/admin/namespaces/{namespace}/codes/campaigns/{campaignId}/disable/bulk

        method: PUT

        tags: ["Campaign"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        campaign_id: (campaignId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        batch_name: (batchName) OPTIONAL str in query

        batch_no: (batchNo) OPTIONAL List[int] in query

    Responses:
        200: OK - BulkOperationResult (successful operation)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = BulkDisableCodes.create(
        campaign_id=campaign_id,
        batch_name=batch_name,
        batch_no=batch_no,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(BulkDisableCodes)
async def bulk_disable_codes_async(
    campaign_id: str,
    batch_name: Optional[str] = None,
    batch_no: Optional[List[int]] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Bulk disable codes (bulkDisableCodes)

    Bulk disable codes.

    Bulk disable campaign codes, all matched codes will be disabled except those have already been redeemed.
    Other detail info:

      * Returns : the number of code actually disabled

    Properties:
        url: /platform/admin/namespaces/{namespace}/codes/campaigns/{campaignId}/disable/bulk

        method: PUT

        tags: ["Campaign"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        campaign_id: (campaignId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        batch_name: (batchName) OPTIONAL str in query

        batch_no: (batchNo) OPTIONAL List[int] in query

    Responses:
        200: OK - BulkOperationResult (successful operation)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = BulkDisableCodes.create(
        campaign_id=campaign_id,
        batch_name=batch_name,
        batch_no=batch_no,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(BulkEnableCodes)
def bulk_enable_codes(
    campaign_id: str,
    batch_name: Optional[str] = None,
    batch_no: Optional[List[int]] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Bulk enable codes (bulkEnableCodes)

    Bulk enable campaign codes.
    Other detail info:

      * Returns : the number of code actually enabled

    Properties:
        url: /platform/admin/namespaces/{namespace}/codes/campaigns/{campaignId}/enable/bulk

        method: PUT

        tags: ["Campaign"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        campaign_id: (campaignId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        batch_name: (batchName) OPTIONAL str in query

        batch_no: (batchNo) OPTIONAL List[int] in query

    Responses:
        200: OK - BulkOperationResult (successful operation)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = BulkEnableCodes.create(
        campaign_id=campaign_id,
        batch_name=batch_name,
        batch_no=batch_no,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(BulkEnableCodes)
async def bulk_enable_codes_async(
    campaign_id: str,
    batch_name: Optional[str] = None,
    batch_no: Optional[List[int]] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Bulk enable codes (bulkEnableCodes)

    Bulk enable campaign codes.
    Other detail info:

      * Returns : the number of code actually enabled

    Properties:
        url: /platform/admin/namespaces/{namespace}/codes/campaigns/{campaignId}/enable/bulk

        method: PUT

        tags: ["Campaign"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        campaign_id: (campaignId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        batch_name: (batchName) OPTIONAL str in query

        batch_no: (batchNo) OPTIONAL List[int] in query

    Responses:
        200: OK - BulkOperationResult (successful operation)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = BulkEnableCodes.create(
        campaign_id=campaign_id,
        batch_name=batch_name,
        batch_no=batch_no,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(CreateCampaign)
def create_campaign(
    body: CampaignCreate,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Create campaign (createCampaign)

    Create campaign.
    Other detail info:

      * Returns : created campaign

    Properties:
        url: /platform/admin/namespaces/{namespace}/campaigns

        method: POST

        tags: ["Campaign"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED CampaignCreate in body

        namespace: (namespace) REQUIRED str in path

    Responses:
        201: Created - CampaignInfo (successful operation)

        400: Bad Request - ErrorEntity (37121: Invalid currency namespace [{namespace}] in discount config: {tips})

        409: Conflict - ErrorEntity (37171: Campaign [{name}] already exists in namespace [{namespace}])

        422: Unprocessable Entity - ValidationErrorEntity (20002: validation error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = CreateCampaign.create(
        body=body,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(CreateCampaign)
async def create_campaign_async(
    body: CampaignCreate,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Create campaign (createCampaign)

    Create campaign.
    Other detail info:

      * Returns : created campaign

    Properties:
        url: /platform/admin/namespaces/{namespace}/campaigns

        method: POST

        tags: ["Campaign"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED CampaignCreate in body

        namespace: (namespace) REQUIRED str in path

    Responses:
        201: Created - CampaignInfo (successful operation)

        400: Bad Request - ErrorEntity (37121: Invalid currency namespace [{namespace}] in discount config: {tips})

        409: Conflict - ErrorEntity (37171: Campaign [{name}] already exists in namespace [{namespace}])

        422: Unprocessable Entity - ValidationErrorEntity (20002: validation error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = CreateCampaign.create(
        body=body,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(CreateCodes)
def create_codes(
    body: CodeCreate,
    campaign_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Create campaign codes (createCodes)

    This API is used to create campaign codes, it will increase the batch No. based on last creation.
    Other detail info:

      * Returns : number of codes created

    Properties:
        url: /platform/admin/namespaces/{namespace}/codes/campaigns/{campaignId}

        method: POST

        tags: ["Campaign"]

        consumes: ["application/json"]

        produces: []

        securities: [BEARER_AUTH]

        body: (body) REQUIRED CodeCreate in body

        campaign_id: (campaignId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        201: Created - CodeCreateResult (successful operation)

        404: Not Found - ErrorEntity (37141: Campaign [{campaignId}] does not exist in namespace [{namespace}])

        409: Conflict - ErrorEntity (37180: Code [{code}] already exists in namespace [{namespace}])

        422: Unprocessable Entity - ValidationErrorEntity (20002: validation error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = CreateCodes.create(
        body=body,
        campaign_id=campaign_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(CreateCodes)
async def create_codes_async(
    body: CodeCreate,
    campaign_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Create campaign codes (createCodes)

    This API is used to create campaign codes, it will increase the batch No. based on last creation.
    Other detail info:

      * Returns : number of codes created

    Properties:
        url: /platform/admin/namespaces/{namespace}/codes/campaigns/{campaignId}

        method: POST

        tags: ["Campaign"]

        consumes: ["application/json"]

        produces: []

        securities: [BEARER_AUTH]

        body: (body) REQUIRED CodeCreate in body

        campaign_id: (campaignId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        201: Created - CodeCreateResult (successful operation)

        404: Not Found - ErrorEntity (37141: Campaign [{campaignId}] does not exist in namespace [{namespace}])

        409: Conflict - ErrorEntity (37180: Code [{code}] already exists in namespace [{namespace}])

        422: Unprocessable Entity - ValidationErrorEntity (20002: validation error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = CreateCodes.create(
        body=body,
        campaign_id=campaign_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(DisableCode)
def disable_code(
    code: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Disable code (disableCode)

    Disable code.

    Disable an active code, the code can't be disabled if it has already been redeemed.
    Other detail info:

      * Returns : disabled code

    Properties:
        url: /platform/admin/namespaces/{namespace}/codes/{code}/disable

        method: PUT

        tags: ["Campaign"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        code: (code) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - CodeInfo (successful operation)

        404: Not Found - ErrorEntity (37142: Code [{code}] does not exist in namespace [{namespace}] | 37176: Code [{code}] has been redeemed in namespace [{namespace}])
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = DisableCode.create(
        code=code,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(DisableCode)
async def disable_code_async(
    code: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Disable code (disableCode)

    Disable code.

    Disable an active code, the code can't be disabled if it has already been redeemed.
    Other detail info:

      * Returns : disabled code

    Properties:
        url: /platform/admin/namespaces/{namespace}/codes/{code}/disable

        method: PUT

        tags: ["Campaign"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        code: (code) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - CodeInfo (successful operation)

        404: Not Found - ErrorEntity (37142: Code [{code}] does not exist in namespace [{namespace}] | 37176: Code [{code}] has been redeemed in namespace [{namespace}])
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = DisableCode.create(
        code=code,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(Download)
def download(
    campaign_id: str,
    batch_name: Optional[str] = None,
    batch_no: Optional[List[int]] = None,
    with_batch_name: Optional[bool] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Download codes (download)

    Download all or a batch of campaign's codes as a csv file.
    Other detail info:

      * Returns : codes csv file
      * The csv file will always have Batch Name column, but this column will be filled only when the withBatchName parameter is true , or when the batchName filter is not blank.

    Properties:
        url: /platform/admin/namespaces/{namespace}/codes/campaigns/{campaignId}/codes.csv

        method: GET

        tags: ["Campaign"]

        consumes: []

        produces: ["text/csv"]

        securities: [BEARER_AUTH]

        campaign_id: (campaignId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        batch_name: (batchName) OPTIONAL str in query

        batch_no: (batchNo) OPTIONAL List[int] in query

        with_batch_name: (withBatchName) OPTIONAL bool in query

    Responses:
        200: OK - Any (Successful operation)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = Download.create(
        campaign_id=campaign_id,
        batch_name=batch_name,
        batch_no=batch_no,
        with_batch_name=with_batch_name,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(Download)
async def download_async(
    campaign_id: str,
    batch_name: Optional[str] = None,
    batch_no: Optional[List[int]] = None,
    with_batch_name: Optional[bool] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Download codes (download)

    Download all or a batch of campaign's codes as a csv file.
    Other detail info:

      * Returns : codes csv file
      * The csv file will always have Batch Name column, but this column will be filled only when the withBatchName parameter is true , or when the batchName filter is not blank.

    Properties:
        url: /platform/admin/namespaces/{namespace}/codes/campaigns/{campaignId}/codes.csv

        method: GET

        tags: ["Campaign"]

        consumes: []

        produces: ["text/csv"]

        securities: [BEARER_AUTH]

        campaign_id: (campaignId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        batch_name: (batchName) OPTIONAL str in query

        batch_no: (batchNo) OPTIONAL List[int] in query

        with_batch_name: (withBatchName) OPTIONAL bool in query

    Responses:
        200: OK - Any (Successful operation)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = Download.create(
        campaign_id=campaign_id,
        batch_name=batch_name,
        batch_no=batch_no,
        with_batch_name=with_batch_name,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(EnableCode)
def enable_code(
    code: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Enable code (enableCode)

    Enable code.
    Other detail info:

      * Returns : enabled code

    Properties:
        url: /platform/admin/namespaces/{namespace}/codes/{code}/enable

        method: PUT

        tags: ["Campaign"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        code: (code) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - CodeInfo (successful operation)

        404: Not Found - ErrorEntity (37142: Code [{code}] does not exist in namespace [{namespace}])
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = EnableCode.create(
        code=code,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(EnableCode)
async def enable_code_async(
    code: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Enable code (enableCode)

    Enable code.
    Other detail info:

      * Returns : enabled code

    Properties:
        url: /platform/admin/namespaces/{namespace}/codes/{code}/enable

        method: PUT

        tags: ["Campaign"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        code: (code) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - CodeInfo (successful operation)

        404: Not Found - ErrorEntity (37142: Code [{code}] does not exist in namespace [{namespace}])
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = EnableCode.create(
        code=code,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(GetCampaign)
def get_campaign(
    campaign_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get campaign (getCampaign)

    Get campaign info.
    Other detail info:

      * Returns : campaign info

    Properties:
        url: /platform/admin/namespaces/{namespace}/campaigns/{campaignId}

        method: GET

        tags: ["Campaign"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        campaign_id: (campaignId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - CampaignInfo (successful operation)

        404: Not Found - ErrorEntity (37141: Campaign [{campaignId}] does not exist in namespace [{namespace}])
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetCampaign.create(
        campaign_id=campaign_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(GetCampaign)
async def get_campaign_async(
    campaign_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get campaign (getCampaign)

    Get campaign info.
    Other detail info:

      * Returns : campaign info

    Properties:
        url: /platform/admin/namespaces/{namespace}/campaigns/{campaignId}

        method: GET

        tags: ["Campaign"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        campaign_id: (campaignId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - CampaignInfo (successful operation)

        404: Not Found - ErrorEntity (37141: Campaign [{campaignId}] does not exist in namespace [{namespace}])
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetCampaign.create(
        campaign_id=campaign_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(GetCampaignDynamic)
def get_campaign_dynamic(
    campaign_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get campaign dynamic (getCampaignDynamic)

    Get campaign dynamic.
    Other detail info:

      * Returns : campaign dynamic

    Properties:
        url: /platform/admin/namespaces/{namespace}/campaigns/{campaignId}/dynamic

        method: GET

        tags: ["Campaign"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        campaign_id: (campaignId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - CampaignDynamicInfo (successful operation)

        404: Not Found - ErrorEntity (37141: Campaign [{campaignId}] does not exist in namespace [{namespace}])
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetCampaignDynamic.create(
        campaign_id=campaign_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(GetCampaignDynamic)
async def get_campaign_dynamic_async(
    campaign_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get campaign dynamic (getCampaignDynamic)

    Get campaign dynamic.
    Other detail info:

      * Returns : campaign dynamic

    Properties:
        url: /platform/admin/namespaces/{namespace}/campaigns/{campaignId}/dynamic

        method: GET

        tags: ["Campaign"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        campaign_id: (campaignId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - CampaignDynamicInfo (successful operation)

        404: Not Found - ErrorEntity (37141: Campaign [{campaignId}] does not exist in namespace [{namespace}])
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetCampaignDynamic.create(
        campaign_id=campaign_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(GetCode)
def get_code(
    code: str,
    redeemable: Optional[bool] = None,
    with_batch_name: Optional[bool] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get code info (getCode)

    Get campaign code, it will check code whether available to redeem if redeemable true.
    Other detail info:

      * Returns : code info

    Properties:
        url: /platform/admin/namespaces/{namespace}/codes/{code}

        method: GET

        tags: ["Campaign"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        code: (code) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        redeemable: (redeemable) OPTIONAL bool in query

        with_batch_name: (withBatchName) OPTIONAL bool in query

    Responses:
        200: OK - CodeInfo (successful operation)

        404: Not Found - ErrorEntity (37142: Code [{code}] does not exist in namespace [{namespace}] | 37144: Campaign batch name does not exist for batch number [{batchNo}] campaign [{campaignId}] in namespace [{namespace}].)

        409: Conflict - ErrorEntity (37172: Campaign [{campaignId}] is inactive in namespace [{namespace}] | 37173: Code [{code}] is inactive in namespace [{namespace}] | 37174: Exceeded max redeem count per code [{maxCount}] | 37177: Code redemption not started | 37178: Code redemption already ended)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetCode.create(
        code=code,
        redeemable=redeemable,
        with_batch_name=with_batch_name,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(GetCode)
async def get_code_async(
    code: str,
    redeemable: Optional[bool] = None,
    with_batch_name: Optional[bool] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get code info (getCode)

    Get campaign code, it will check code whether available to redeem if redeemable true.
    Other detail info:

      * Returns : code info

    Properties:
        url: /platform/admin/namespaces/{namespace}/codes/{code}

        method: GET

        tags: ["Campaign"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        code: (code) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        redeemable: (redeemable) OPTIONAL bool in query

        with_batch_name: (withBatchName) OPTIONAL bool in query

    Responses:
        200: OK - CodeInfo (successful operation)

        404: Not Found - ErrorEntity (37142: Code [{code}] does not exist in namespace [{namespace}] | 37144: Campaign batch name does not exist for batch number [{batchNo}] campaign [{campaignId}] in namespace [{namespace}].)

        409: Conflict - ErrorEntity (37172: Campaign [{campaignId}] is inactive in namespace [{namespace}] | 37173: Code [{code}] is inactive in namespace [{namespace}] | 37174: Exceeded max redeem count per code [{maxCount}] | 37177: Code redemption not started | 37178: Code redemption already ended)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetCode.create(
        code=code,
        redeemable=redeemable,
        with_batch_name=with_batch_name,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(QueryCampaignBatchNames)
def query_campaign_batch_names(
    campaign_id: str,
    batch_name: Optional[str] = None,
    limit: Optional[int] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Query campaign batch names (queryCampaignBatchNames)

    Query campaign batch name by fuzzy match.
    Other detail info:

      * Returns : list of campaign batch names

    Properties:
        url: /platform/admin/namespaces/{namespace}/campaigns/{campaignId}/batchNames

        method: GET

        tags: ["Campaign"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        campaign_id: (campaignId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        batch_name: (batchName) OPTIONAL str in query

        limit: (limit) OPTIONAL int in query

    Responses:
        200: OK - List[CampaignBatchNameInfo] (successful operation)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = QueryCampaignBatchNames.create(
        campaign_id=campaign_id,
        batch_name=batch_name,
        limit=limit,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(QueryCampaignBatchNames)
async def query_campaign_batch_names_async(
    campaign_id: str,
    batch_name: Optional[str] = None,
    limit: Optional[int] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Query campaign batch names (queryCampaignBatchNames)

    Query campaign batch name by fuzzy match.
    Other detail info:

      * Returns : list of campaign batch names

    Properties:
        url: /platform/admin/namespaces/{namespace}/campaigns/{campaignId}/batchNames

        method: GET

        tags: ["Campaign"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        campaign_id: (campaignId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        batch_name: (batchName) OPTIONAL str in query

        limit: (limit) OPTIONAL int in query

    Responses:
        200: OK - List[CampaignBatchNameInfo] (successful operation)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = QueryCampaignBatchNames.create(
        campaign_id=campaign_id,
        batch_name=batch_name,
        limit=limit,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(QueryCampaigns)
def query_campaigns(
    limit: Optional[int] = None,
    name: Optional[str] = None,
    offset: Optional[int] = None,
    tag: Optional[str] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Query campaigns (queryCampaigns)

    Query campaigns, if name is presented, it's fuzzy match.
    Other detail info:

      * Returns : slice of campaigns

    Properties:
        url: /platform/admin/namespaces/{namespace}/campaigns

        method: GET

        tags: ["Campaign"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        limit: (limit) OPTIONAL int in query

        name: (name) OPTIONAL str in query

        offset: (offset) OPTIONAL int in query

        tag: (tag) OPTIONAL str in query

    Responses:
        200: OK - CampaignPagingSlicedResult (successful operation)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = QueryCampaigns.create(
        limit=limit,
        name=name,
        offset=offset,
        tag=tag,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(QueryCampaigns)
async def query_campaigns_async(
    limit: Optional[int] = None,
    name: Optional[str] = None,
    offset: Optional[int] = None,
    tag: Optional[str] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Query campaigns (queryCampaigns)

    Query campaigns, if name is presented, it's fuzzy match.
    Other detail info:

      * Returns : slice of campaigns

    Properties:
        url: /platform/admin/namespaces/{namespace}/campaigns

        method: GET

        tags: ["Campaign"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        limit: (limit) OPTIONAL int in query

        name: (name) OPTIONAL str in query

        offset: (offset) OPTIONAL int in query

        tag: (tag) OPTIONAL str in query

    Responses:
        200: OK - CampaignPagingSlicedResult (successful operation)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = QueryCampaigns.create(
        limit=limit,
        name=name,
        offset=offset,
        tag=tag,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(QueryCodes)
def query_codes(
    campaign_id: str,
    active_only: Optional[bool] = None,
    batch_name: Optional[str] = None,
    batch_no: Optional[List[int]] = None,
    code: Optional[str] = None,
    limit: Optional[int] = None,
    offset: Optional[int] = None,
    with_batch_name: Optional[bool] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Query codes (queryCodes)

    Query campaign codes.
    Other detail info:

      * Returns : list of codes
      * The batchName field in the codes response will be present only when the withBatchName parameter is true , or when the batchName filter is not blank.

    Properties:
        url: /platform/admin/namespaces/{namespace}/codes/campaigns/{campaignId}

        method: GET

        tags: ["Campaign"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        campaign_id: (campaignId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        active_only: (activeOnly) OPTIONAL bool in query

        batch_name: (batchName) OPTIONAL str in query

        batch_no: (batchNo) OPTIONAL List[int] in query

        code: (code) OPTIONAL str in query

        limit: (limit) OPTIONAL int in query

        offset: (offset) OPTIONAL int in query

        with_batch_name: (withBatchName) OPTIONAL bool in query

    Responses:
        200: OK - CodeInfoPagingSlicedResult (successful operation)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = QueryCodes.create(
        campaign_id=campaign_id,
        active_only=active_only,
        batch_name=batch_name,
        batch_no=batch_no,
        code=code,
        limit=limit,
        offset=offset,
        with_batch_name=with_batch_name,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(QueryCodes)
async def query_codes_async(
    campaign_id: str,
    active_only: Optional[bool] = None,
    batch_name: Optional[str] = None,
    batch_no: Optional[List[int]] = None,
    code: Optional[str] = None,
    limit: Optional[int] = None,
    offset: Optional[int] = None,
    with_batch_name: Optional[bool] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Query codes (queryCodes)

    Query campaign codes.
    Other detail info:

      * Returns : list of codes
      * The batchName field in the codes response will be present only when the withBatchName parameter is true , or when the batchName filter is not blank.

    Properties:
        url: /platform/admin/namespaces/{namespace}/codes/campaigns/{campaignId}

        method: GET

        tags: ["Campaign"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        campaign_id: (campaignId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        active_only: (activeOnly) OPTIONAL bool in query

        batch_name: (batchName) OPTIONAL str in query

        batch_no: (batchNo) OPTIONAL List[int] in query

        code: (code) OPTIONAL str in query

        limit: (limit) OPTIONAL int in query

        offset: (offset) OPTIONAL int in query

        with_batch_name: (withBatchName) OPTIONAL bool in query

    Responses:
        200: OK - CodeInfoPagingSlicedResult (successful operation)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = QueryCodes.create(
        campaign_id=campaign_id,
        active_only=active_only,
        batch_name=batch_name,
        batch_no=batch_no,
        code=code,
        limit=limit,
        offset=offset,
        with_batch_name=with_batch_name,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(QueryRedeemHistory)
def query_redeem_history(
    campaign_id: str,
    code: Optional[str] = None,
    limit: Optional[int] = None,
    offset: Optional[int] = None,
    user_id: Optional[str] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Query redeem history (queryRedeemHistory)

    Query redeem history.
    Other detail info:

      * Returns : slice of redeem history

    Properties:
        url: /platform/admin/namespaces/{namespace}/codes/campaigns/{campaignId}/history

        method: GET

        tags: ["Campaign"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        campaign_id: (campaignId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        code: (code) OPTIONAL str in query

        limit: (limit) OPTIONAL int in query

        offset: (offset) OPTIONAL int in query

        user_id: (userId) OPTIONAL str in query

    Responses:
        200: OK - RedeemHistoryPagingSlicedResult (successful operation)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = QueryRedeemHistory.create(
        campaign_id=campaign_id,
        code=code,
        limit=limit,
        offset=offset,
        user_id=user_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(QueryRedeemHistory)
async def query_redeem_history_async(
    campaign_id: str,
    code: Optional[str] = None,
    limit: Optional[int] = None,
    offset: Optional[int] = None,
    user_id: Optional[str] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Query redeem history (queryRedeemHistory)

    Query redeem history.
    Other detail info:

      * Returns : slice of redeem history

    Properties:
        url: /platform/admin/namespaces/{namespace}/codes/campaigns/{campaignId}/history

        method: GET

        tags: ["Campaign"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        campaign_id: (campaignId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        code: (code) OPTIONAL str in query

        limit: (limit) OPTIONAL int in query

        offset: (offset) OPTIONAL int in query

        user_id: (userId) OPTIONAL str in query

    Responses:
        200: OK - RedeemHistoryPagingSlicedResult (successful operation)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = QueryRedeemHistory.create(
        campaign_id=campaign_id,
        code=code,
        limit=limit,
        offset=offset,
        user_id=user_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(RenameBatch)
def rename_batch(
    body: CampaignBatchNameChange,
    campaign_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Rename batch (renameBatch)

    Change campaign batch name.
    Other detail info:

    Properties:
        url: /platform/admin/namespaces/{namespace}/campaigns/{campaignId}/batchName

        method: PUT

        tags: ["Campaign"]

        consumes: ["application/json"]

        produces: []

        securities: [BEARER_AUTH]

        body: (body) REQUIRED CampaignBatchNameChange in body

        campaign_id: (campaignId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        204: No Content - (Batch was renamed successfully)

        404: Not Found - ErrorEntity (37141: Campaign [{campaignId}] does not exist in namespace [{namespace}] | 37143: Batch name [{batchName}] does not exist for campaign [{campaignId}] in namespace [{namespace}].)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = RenameBatch.create(
        body=body,
        campaign_id=campaign_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(RenameBatch)
async def rename_batch_async(
    body: CampaignBatchNameChange,
    campaign_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Rename batch (renameBatch)

    Change campaign batch name.
    Other detail info:

    Properties:
        url: /platform/admin/namespaces/{namespace}/campaigns/{campaignId}/batchName

        method: PUT

        tags: ["Campaign"]

        consumes: ["application/json"]

        produces: []

        securities: [BEARER_AUTH]

        body: (body) REQUIRED CampaignBatchNameChange in body

        campaign_id: (campaignId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        204: No Content - (Batch was renamed successfully)

        404: Not Found - ErrorEntity (37141: Campaign [{campaignId}] does not exist in namespace [{namespace}] | 37143: Batch name [{batchName}] does not exist for campaign [{campaignId}] in namespace [{namespace}].)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = RenameBatch.create(
        body=body,
        campaign_id=campaign_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(UpdateCampaign)
def update_campaign(
    body: CampaignUpdate,
    campaign_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Update campaign (updateCampaign)

    Update campaign.
    Other detail info:

      * Returns : updated campaign

    Properties:
        url: /platform/admin/namespaces/{namespace}/campaigns/{campaignId}

        method: PUT

        tags: ["Campaign"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED CampaignUpdate in body

        campaign_id: (campaignId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - CampaignInfo (successful operation)

        404: Not Found - ErrorEntity (37141: Campaign [{campaignId}] does not exist in namespace [{namespace}])

        409: Conflict - ErrorEntity (37171: Campaign [{name}] already exists in namespace [{namespace}])

        422: Unprocessable Entity - ValidationErrorEntity (20002: validation error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = UpdateCampaign.create(
        body=body,
        campaign_id=campaign_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(UpdateCampaign)
async def update_campaign_async(
    body: CampaignUpdate,
    campaign_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Update campaign (updateCampaign)

    Update campaign.
    Other detail info:

      * Returns : updated campaign

    Properties:
        url: /platform/admin/namespaces/{namespace}/campaigns/{campaignId}

        method: PUT

        tags: ["Campaign"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED CampaignUpdate in body

        campaign_id: (campaignId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - CampaignInfo (successful operation)

        404: Not Found - ErrorEntity (37141: Campaign [{campaignId}] does not exist in namespace [{namespace}])

        409: Conflict - ErrorEntity (37171: Campaign [{name}] already exists in namespace [{namespace}])

        422: Unprocessable Entity - ValidationErrorEntity (20002: validation error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = UpdateCampaign.create(
        body=body,
        campaign_id=campaign_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Inventory Service

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model
from ....core import StrEnum


class StatusEnum(StrEnum):
    INIT = "INIT"
    TIED = "TIED"


class ApimodelsInventoryConfigurationResp(Model):
    """Apimodels inventory configuration resp (apimodels.InventoryConfigurationResp)

    Properties:
        code: (code) REQUIRED str

        created_at: (createdAt) REQUIRED str

        description: (description) REQUIRED str

        id_: (id) REQUIRED str

        initial_max_slots: (initialMaxSlots) REQUIRED int

        max_instances_per_user: (maxInstancesPerUser) REQUIRED int

        max_upgrade_slots: (maxUpgradeSlots) REQUIRED int

        name: (name) REQUIRED str

        namespace: (namespace) REQUIRED str

        status: (status) REQUIRED Union[str, StatusEnum]

        updated_at: (updatedAt) REQUIRED str
    """

    # region fields

    code: str  # REQUIRED
    created_at: str  # REQUIRED
    description: str  # REQUIRED
    id_: str  # REQUIRED
    initial_max_slots: int  # REQUIRED
    max_instances_per_user: int  # REQUIRED
    max_upgrade_slots: int  # REQUIRED
    name: str  # REQUIRED
    namespace: str  # REQUIRED
    status: Union[str, StatusEnum]  # REQUIRED
    updated_at: str  # REQUIRED

    # endregion fields

    # region with_x methods

    def with_code(self, value: str) -> ApimodelsInventoryConfigurationResp:
        self.code = value
        return self

    def with_created_at(self, value: str) -> ApimodelsInventoryConfigurationResp:
        self.created_at = value
        return self

    def with_description(self, value: str) -> ApimodelsInventoryConfigurationResp:
        self.description = value
        return self

    def with_id(self, value: str) -> ApimodelsInventoryConfigurationResp:
        self.id_ = value
        return self

    def with_initial_max_slots(self, value: int) -> ApimodelsInventoryConfigurationResp:
        self.initial_max_slots = value
        return self

    def with_max_instances_per_user(
        self, value: int
    ) -> ApimodelsInventoryConfigurationResp:
        self.max_instances_per_user = value
        return self

    def with_max_upgrade_slots(self, value: int) -> ApimodelsInventoryConfigurationResp:
        self.max_upgrade_slots = value
        return self

    def with_name(self, value: str) -> ApimodelsInventoryConfigurationResp:
        self.name = value
        return self

    def with_namespace(self, value: str) -> ApimodelsInventoryConfigurationResp:
        self.namespace = value
        return self

    def with_status(
        self, value: Union[str, StatusEnum]
    ) -> ApimodelsInventoryConfigurationResp:
        self.status = value
        return self

    def with_updated_at(self, value: str) -> ApimodelsInventoryConfigurationResp:
        self.updated_at = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "code"):
            result["code"] = str(self.code)
        elif include_empty:
            result["code"] = ""
        if hasattr(self, "created_at"):
            result["createdAt"] = str(self.created_at)
        elif include_empty:
            result["createdAt"] = ""
        if hasattr(self, "description"):
            result["description"] = str(self.description)
        elif include_empty:
            result["description"] = ""
        if hasattr(self, "id_"):
            result["id"] = str(self.id_)
        elif include_empty:
            result["id"] = ""
        if hasattr(self, "initial_max_slots"):
            result["initialMaxSlots"] = int(self.initial_max_slots)
        elif include_empty:
            result["initialMaxSlots"] = 0
        if hasattr(self, "max_instances_per_user"):
            result["maxInstancesPerUser"] = int(self.max_instances_per_user)
        elif include_empty:
            result["maxInstancesPerUser"] = 0
        if hasattr(self, "max_upgrade_slots"):
            result["maxUpgradeSlots"] = int(self.max_upgrade_slots)
        elif include_empty:
            result["maxUpgradeSlots"] = 0
        if hasattr(self, "name"):
            result["name"] = str(self.name)
        elif include_empty:
            result["name"] = ""
        if hasattr(self, "namespace"):
            result["namespace"] = str(self.namespace)
        elif include_empty:
            result["namespace"] = ""
        if hasattr(self, "status"):
            result["status"] = str(self.status)
        elif include_empty:
            result["status"] = Union[str, StatusEnum]()
        if hasattr(self, "updated_at"):
            result["updatedAt"] = str(self.updated_at)
        elif include_empty:
            result["updatedAt"] = ""
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        code: str,
        created_at: str,
        description: str,
        id_: str,
        initial_max_slots: int,
        max_instances_per_user: int,
        max_upgrade_slots: int,
        name: str,
        namespace: str,
        status: Union[str, StatusEnum],
        updated_at: str,
        **kwargs,
    ) -> ApimodelsInventoryConfigurationResp:
        instance = cls()
        instance.code = code
        instance.created_at = created_at
        instance.description = description
        instance.id_ = id_
        instance.initial_max_slots = initial_max_slots
        instance.max_instances_per_user = max_instances_per_user
        instance.max_upgrade_slots = max_upgrade_slots
        instance.name = name
        instance.namespace = namespace
        instance.status = status
        instance.updated_at = updated_at
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> ApimodelsInventoryConfigurationResp:
        instance = cls()
        if not dict_:
            return instance
        if "code" in dict_ and dict_["code"] is not None:
            instance.code = str(dict_["code"])
        elif include_empty:
            instance.code = ""
        if "createdAt" in dict_ and dict_["createdAt"] is not None:
            instance.created_at = str(dict_["createdAt"])
        elif include_empty:
            instance.created_at = ""
        if "description" in dict_ and dict_["description"] is not None:
            instance.description = str(dict_["description"])
        elif include_empty:
            instance.description = ""
        if "id" in dict_ and dict_["id"] is not None:
            instance.id_ = str(dict_["id"])
        elif include_empty:
            instance.id_ = ""
        if "initialMaxSlots" in dict_ and dict_["initialMaxSlots"] is not None:
            instance.initial_max_slots = int(dict_["initialMaxSlots"])
        elif include_empty:
            instance.initial_max_slots = 0
        if "maxInstancesPerUser" in dict_ and dict_["maxInstancesPerUser"] is not None:
            instance.max_instances_per_user = int(dict_["maxInstancesPerUser"])
        elif include_empty:
            instance.max_instances_per_user = 0
        if "maxUpgradeSlots" in dict_ and dict_["maxUpgradeSlots"] is not None:
            instance.max_upgrade_slots = int(dict_["maxUpgradeSlots"])
        elif include_empty:
            instance.max_upgrade_slots = 0
        if "name" in dict_ and dict_["name"] is not None:
            instance.name = str(dict_["name"])
        elif include_empty:
            instance.name = ""
        if "namespace" in dict_ and dict_["namespace"] is not None:
            instance.namespace = str(dict_["namespace"])
        elif include_empty:
            instance.namespace = ""
        if "status" in dict_ and dict_["status"] is not None:
            instance.status = str(dict_["status"])
        elif include_empty:
            instance.status = Union[str, StatusEnum]()
        if "updatedAt" in dict_ and dict_["updatedAt"] is not None:
            instance.updated_at = str(dict_["updatedAt"])
        elif include_empty:
            instance.updated_at = ""
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, ApimodelsInventoryConfigurationResp]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[ApimodelsInventoryConfigurationResp]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        ApimodelsInventoryConfigurationResp,
        List[ApimodelsInventoryConfigurationResp],
        Dict[Any, ApimodelsInventoryConfigurationResp],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "code": "code",
            "createdAt": "created_at",
            "description": "description",
            "id": "id_",
            "initialMaxSlots": "initial_max_slots",
            "maxInstancesPerUser": "max_instances_per_user",
            "maxUpgradeSlots": "max_upgrade_slots",
            "name": "name",
            "namespace": "namespace",
            "status": "status",
            "updatedAt": "updated_at",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "code": True,
            "createdAt": True,
            "description": True,
            "id": True,
            "initialMaxSlots": True,
            "maxInstancesPerUser": True,
            "maxUpgradeSlots": True,
            "name": True,
            "namespace": True,
            "status": True,
            "updatedAt": True,
        }

    @staticmethod
    def get_enum_map() -> Dict[str, List[Any]]:
        return {
            "status": ["INIT", "TIED"],
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# Fleet Commander

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model


class ApiImageUpdate(Model):
    """Api image update (api.ImageUpdate)

    Properties:
        added_tags: (addedTags) REQUIRED List[str]

        is_protected: (isProtected) REQUIRED bool

        name: (name) REQUIRED str

        removed_tags: (removedTags) REQUIRED List[str]
    """

    # region fields

    added_tags: List[str]  # REQUIRED
    is_protected: bool  # REQUIRED
    name: str  # REQUIRED
    removed_tags: List[str]  # REQUIRED

    # endregion fields

    # region with_x methods

    def with_added_tags(self, value: List[str]) -> ApiImageUpdate:
        self.added_tags = value
        return self

    def with_is_protected(self, value: bool) -> ApiImageUpdate:
        self.is_protected = value
        return self

    def with_name(self, value: str) -> ApiImageUpdate:
        self.name = value
        return self

    def with_removed_tags(self, value: List[str]) -> ApiImageUpdate:
        self.removed_tags = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "added_tags"):
            result["addedTags"] = [str(i0) for i0 in self.added_tags]
        elif include_empty:
            result["addedTags"] = []
        if hasattr(self, "is_protected"):
            result["isProtected"] = bool(self.is_protected)
        elif include_empty:
            result["isProtected"] = False
        if hasattr(self, "name"):
            result["name"] = str(self.name)
        elif include_empty:
            result["name"] = ""
        if hasattr(self, "removed_tags"):
            result["removedTags"] = [str(i0) for i0 in self.removed_tags]
        elif include_empty:
            result["removedTags"] = []
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        added_tags: List[str],
        is_protected: bool,
        name: str,
        removed_tags: List[str],
        **kwargs,
    ) -> ApiImageUpdate:
        instance = cls()
        instance.added_tags = added_tags
        instance.is_protected = is_protected
        instance.name = name
        instance.removed_tags = removed_tags
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> ApiImageUpdate:
        instance = cls()
        if not dict_:
            return instance
        if "addedTags" in dict_ and dict_["addedTags"] is not None:
            instance.added_tags = [str(i0) for i0 in dict_["addedTags"]]
        elif include_empty:
            instance.added_tags = []
        if "isProtected" in dict_ and dict_["isProtected"] is not None:
            instance.is_protected = bool(dict_["isProtected"])
        elif include_empty:
            instance.is_protected = False
        if "name" in dict_ and dict_["name"] is not None:
            instance.name = str(dict_["name"])
        elif include_empty:
            instance.name = ""
        if "removedTags" in dict_ and dict_["removedTags"] is not None:
            instance.removed_tags = [str(i0) for i0 in dict_["removedTags"]]
        elif include_empty:
            instance.removed_tags = []
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, ApiImageUpdate]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[ApiImageUpdate]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[ApiImageUpdate, List[ApiImageUpdate], Dict[Any, ApiImageUpdate]]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "addedTags": "added_tags",
            "isProtected": "is_protected",
            "name": "name",
            "removedTags": "removed_tags",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "addedTags": True,
            "isProtected": True,
            "name": True,
            "removedTags": True,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import HeaderStr
from ....core import get_namespace as get_services_namespace
from ....core import run_request
from ....core import run_request_async
from ....core import same_doc_as

from ..models import CreateBasePolicyRequestV2
from ..models import CreateBasePolicyResponse
from ..models import CreatePolicyRequest
from ..models import CreatePolicyResponse
from ..models import ErrorEntity
from ..models import RetrieveBasePolicyResponse
from ..models import RetrievePoliciesFromBasePolicyResponse
from ..models import RetrievePolicyResponse
from ..models import RetrievePolicyTypeResponse
from ..models import UpdateBasePolicyRequestV2
from ..models import UpdateBasePolicyResponse
from ..models import ValidationErrorEntity

from ..operations.base_legal_policies_with_namespace import CreatePolicy1
from ..operations.base_legal_policies_with_namespace import CreatePolicyUnderBasePolicy
from ..operations.base_legal_policies_with_namespace import DeleteBasePolicy
from ..operations.base_legal_policies_with_namespace import PartialUpdatePolicy1
from ..operations.base_legal_policies_with_namespace import (
    RetrieveAllLegalPoliciesByNamespace,
)
from ..operations.base_legal_policies_with_namespace import (
    RetrieveAllPoliciesFromBasePolicy,
)
from ..operations.base_legal_policies_with_namespace import RetrieveAllPolicyTypes1
from ..operations.base_legal_policies_with_namespace import RetrievePolicyCountry1
from ..operations.base_legal_policies_with_namespace import RetrieveSinglePolicy1
from ..models import CreateBasePolicyRequestV2CountryTypeEnum
from ..models import CreateBasePolicyResponseCountryTypeEnum
from ..models import CreatePolicyRequestCountryTypeEnum
from ..models import UpdateBasePolicyRequestV2CountryTypeEnum


@same_doc_as(CreatePolicy1)
def create_policy_1(
    body: Optional[CreateBasePolicyRequestV2] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Create a Base Legal Policy (createPolicy_1)

    Create a legal policy.
    Note:


      *  countryType field only accept COUNTRY or COUNTRY_GROUP.
      * COUNTRY_GROUP countryType requires the countryGroupName and affectedCountries fields to be provided.


    * policy with COUNTRY_GROUP type include multiple countries and apply the same policy across the entire list of countries

    Properties:
        url: /agreement/admin/namespaces/{namespace}/base-policies

        method: POST

        tags: ["Base Legal Policies With Namespace"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) OPTIONAL CreateBasePolicyRequestV2 in body

        namespace: (namespace) REQUIRED str in path

    Responses:
        201: Created - CreateBasePolicyResponse (successful operation)

        400: Bad Request - ErrorEntity (40032: errors.net.accelbyte.platform.legal.base_policy_namespace_not_match | 40026: errors.net.accelbyte.platform.legal.not_allow_create_studio_policy)

        409: Conflict - ErrorEntity (40030: errors.net.accelbyte.platform.legal.invalid_policy_type)

        422: Unprocessable Entity - ValidationErrorEntity (20002: validation error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = CreatePolicy1.create(
        body=body,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(CreatePolicy1)
async def create_policy_1_async(
    body: Optional[CreateBasePolicyRequestV2] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Create a Base Legal Policy (createPolicy_1)

    Create a legal policy.
    Note:


      *  countryType field only accept COUNTRY or COUNTRY_GROUP.
      * COUNTRY_GROUP countryType requires the countryGroupName and affectedCountries fields to be provided.


    * policy with COUNTRY_GROUP type include multiple countries and apply the same policy across the entire list of countries

    Properties:
        url: /agreement/admin/namespaces/{namespace}/base-policies

        method: POST

        tags: ["Base Legal Policies With Namespace"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) OPTIONAL CreateBasePolicyRequestV2 in body

        namespace: (namespace) REQUIRED str in path

    Responses:
        201: Created - CreateBasePolicyResponse (successful operation)

        400: Bad Request - ErrorEntity (40032: errors.net.accelbyte.platform.legal.base_policy_namespace_not_match | 40026: errors.net.accelbyte.platform.legal.not_allow_create_studio_policy)

        409: Conflict - ErrorEntity (40030: errors.net.accelbyte.platform.legal.invalid_policy_type)

        422: Unprocessable Entity - ValidationErrorEntity (20002: validation error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = CreatePolicy1.create(
        body=body,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(CreatePolicyUnderBasePolicy)
def create_policy_under_base_policy(
    base_policy_id: str,
    body: Optional[CreatePolicyRequest] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Create policy under base policy (createPolicyUnderBasePolicy)

    Create policy under base policy.
    Note:


      *  countryType field only accept COUNTRY or COUNTRY_GROUP.
      * COUNTRY_GROUP policy requires the countryGroupName and countries fields to be provided.


    * policy with COUNTRY_GROUP type include multiple countries and apply the same policy across the entire list of countries

    Properties:
        url: /agreement/admin/namespaces/{namespace}/base-policies/{basePolicyId}/policies

        method: POST

        tags: ["Base Legal Policies With Namespace"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) OPTIONAL CreatePolicyRequest in body

        base_policy_id: (basePolicyId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        201: Created - CreatePolicyResponse (successful operation)

        400: Bad Request - ErrorEntity (40032: errors.net.accelbyte.platform.legal.invalid_base_policy)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = CreatePolicyUnderBasePolicy.create(
        base_policy_id=base_policy_id,
        body=body,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(CreatePolicyUnderBasePolicy)
async def create_policy_under_base_policy_async(
    base_policy_id: str,
    body: Optional[CreatePolicyRequest] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Create policy under base policy (createPolicyUnderBasePolicy)

    Create policy under base policy.
    Note:


      *  countryType field only accept COUNTRY or COUNTRY_GROUP.
      * COUNTRY_GROUP policy requires the countryGroupName and countries fields to be provided.


    * policy with COUNTRY_GROUP type include multiple countries and apply the same policy across the entire list of countries

    Properties:
        url: /agreement/admin/namespaces/{namespace}/base-policies/{basePolicyId}/policies

        method: POST

        tags: ["Base Legal Policies With Namespace"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) OPTIONAL CreatePolicyRequest in body

        base_policy_id: (basePolicyId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        201: Created - CreatePolicyResponse (successful operation)

        400: Bad Request - ErrorEntity (40032: errors.net.accelbyte.platform.legal.invalid_base_policy)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = CreatePolicyUnderBasePolicy.create(
        base_policy_id=base_policy_id,
        body=body,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(DeleteBasePolicy)
def delete_base_policy(
    base_policy_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Delete Base Legal Policy (deleteBasePolicy)

    Delete base legal policy.Can only be deleted if match these criteria:


      * Base policy don't have published policy version
      * Policy version under base policy has never been accepted by any user

    Properties:
        url: /agreement/admin/namespaces/{namespace}/base-policies/{basePolicyId}

        method: DELETE

        tags: ["Base Legal Policies With Namespace"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        base_policy_id: (basePolicyId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        204: No Content - (Successful operation)

        400: Bad Request - ErrorEntity (40032: errors.net.accelbyte.platform.legal.invalid_base_policy | 40068: errors.net.accelbyte.platform.legal.base_policy_have_published_policy_version_and_already_accepted_by_user | 40069: errors.net.accelbyte.platform.legal.base_policy_already_have_published_policy_version)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = DeleteBasePolicy.create(
        base_policy_id=base_policy_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(DeleteBasePolicy)
async def delete_base_policy_async(
    base_policy_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Delete Base Legal Policy (deleteBasePolicy)

    Delete base legal policy.Can only be deleted if match these criteria:


      * Base policy don't have published policy version
      * Policy version under base policy has never been accepted by any user

    Properties:
        url: /agreement/admin/namespaces/{namespace}/base-policies/{basePolicyId}

        method: DELETE

        tags: ["Base Legal Policies With Namespace"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        base_policy_id: (basePolicyId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        204: No Content - (Successful operation)

        400: Bad Request - ErrorEntity (40032: errors.net.accelbyte.platform.legal.invalid_base_policy | 40068: errors.net.accelbyte.platform.legal.base_policy_have_published_policy_version_and_already_accepted_by_user | 40069: errors.net.accelbyte.platform.legal.base_policy_already_have_published_policy_version)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = DeleteBasePolicy.create(
        base_policy_id=base_policy_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(PartialUpdatePolicy1)
def partial_update_policy_1(
    base_policy_id: str,
    body: Optional[UpdateBasePolicyRequestV2] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Update Base Legal Policy (partialUpdatePolicy_1)

    Update an existing base policy.
    Note:


      *  countryType field only accept COUNTRY or COUNTRY_GROUP.
      * COUNTRY_GROUP countryType requires the countryGroupName and affectedCountries fields to be provided.


    * policy with COUNTRY_GROUP type include multiple countries and apply the same policy across the entire list of countries

    Properties:
        url: /agreement/admin/namespaces/{namespace}/base-policies/{basePolicyId}

        method: PATCH

        tags: ["Base Legal Policies With Namespace"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) OPTIONAL UpdateBasePolicyRequestV2 in body

        base_policy_id: (basePolicyId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - UpdateBasePolicyResponse (successful operation)

        400: Bad Request - ErrorEntity (40032: errors.net.accelbyte.platform.legal.invalid_base_policy)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = PartialUpdatePolicy1.create(
        base_policy_id=base_policy_id,
        body=body,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(PartialUpdatePolicy1)
async def partial_update_policy_1_async(
    base_policy_id: str,
    body: Optional[UpdateBasePolicyRequestV2] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Update Base Legal Policy (partialUpdatePolicy_1)

    Update an existing base policy.
    Note:


      *  countryType field only accept COUNTRY or COUNTRY_GROUP.
      * COUNTRY_GROUP countryType requires the countryGroupName and affectedCountries fields to be provided.


    * policy with COUNTRY_GROUP type include multiple countries and apply the same policy across the entire list of countries

    Properties:
        url: /agreement/admin/namespaces/{namespace}/base-policies/{basePolicyId}

        method: PATCH

        tags: ["Base Legal Policies With Namespace"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) OPTIONAL UpdateBasePolicyRequestV2 in body

        base_policy_id: (basePolicyId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - UpdateBasePolicyResponse (successful operation)

        400: Bad Request - ErrorEntity (40032: errors.net.accelbyte.platform.legal.invalid_base_policy)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = PartialUpdatePolicy1.create(
        base_policy_id=base_policy_id,
        body=body,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(RetrieveAllLegalPoliciesByNamespace)
def retrieve_all_legal_policies_by_namespace(
    visible_only: Optional[bool] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Retrieve All Base Legal Policy in the namespace (retrieveAllLegalPoliciesByNamespace)

    Retrieve all base policies in the namespace.

    Properties:
        url: /agreement/admin/namespaces/{namespace}/base-policies

        method: GET

        tags: ["Base Legal Policies With Namespace"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        visible_only: (visibleOnly) OPTIONAL bool in query

    Responses:
        200: OK - List[RetrieveBasePolicyResponse] (successful operation)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = RetrieveAllLegalPoliciesByNamespace.create(
        visible_only=visible_only,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(RetrieveAllLegalPoliciesByNamespace)
async def retrieve_all_legal_policies_by_namespace_async(
    visible_only: Optional[bool] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Retrieve All Base Legal Policy in the namespace (retrieveAllLegalPoliciesByNamespace)

    Retrieve all base policies in the namespace.

    Properties:
        url: /agreement/admin/namespaces/{namespace}/base-policies

        method: GET

        tags: ["Base Legal Policies With Namespace"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        visible_only: (visibleOnly) OPTIONAL bool in query

    Responses:
        200: OK - List[RetrieveBasePolicyResponse] (successful operation)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = RetrieveAllLegalPoliciesByNamespace.create(
        visible_only=visible_only,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(RetrieveAllPoliciesFromBasePolicy)
def retrieve_all_policies_from_base_policy(
    base_policy_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Retrieve all policies from Base Legal Policy (retrieveAllPoliciesFromBasePolicy)

    Retrieve all policies from Base Legal Policy.

    Properties:
        url: /agreement/admin/namespaces/{namespace}/base-policies/{basePolicyId}/policies

        method: GET

        tags: ["Base Legal Policies With Namespace"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        base_policy_id: (basePolicyId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - List[RetrievePoliciesFromBasePolicyResponse] (successful operation)

        404: Not Found - ErrorEntity (40041: errors.net.accelbyte.platform.legal.policy_not_found)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = RetrieveAllPoliciesFromBasePolicy.create(
        base_policy_id=base_policy_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(RetrieveAllPoliciesFromBasePolicy)
async def retrieve_all_policies_from_base_policy_async(
    base_policy_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Retrieve all policies from Base Legal Policy (retrieveAllPoliciesFromBasePolicy)

    Retrieve all policies from Base Legal Policy.

    Properties:
        url: /agreement/admin/namespaces/{namespace}/base-policies/{basePolicyId}/policies

        method: GET

        tags: ["Base Legal Policies With Namespace"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        base_policy_id: (basePolicyId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - List[RetrievePoliciesFromBasePolicyResponse] (successful operation)

        404: Not Found - ErrorEntity (40041: errors.net.accelbyte.platform.legal.policy_not_found)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = RetrieveAllPoliciesFromBasePolicy.create(
        base_policy_id=base_policy_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(RetrieveAllPolicyTypes1)
def retrieve_all_policy_types_1(
    limit: int,
    offset: Optional[int] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Retrieve All Policy Type (retrieveAllPolicyTypes_1)

    Retrieve all supported policy types.

    Properties:
        url: /agreement/admin/namespaces/{namespace}/policy-types

        method: GET

        tags: ["Base Legal Policies With Namespace"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        offset: (offset) OPTIONAL int in query

        limit: (limit) REQUIRED int in query

    Responses:
        200: OK - List[RetrievePolicyTypeResponse] (successful operation)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = RetrieveAllPolicyTypes1.create(
        limit=limit,
        offset=offset,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(RetrieveAllPolicyTypes1)
async def retrieve_all_policy_types_1_async(
    limit: int,
    offset: Optional[int] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Retrieve All Policy Type (retrieveAllPolicyTypes_1)

    Retrieve all supported policy types.

    Properties:
        url: /agreement/admin/namespaces/{namespace}/policy-types

        method: GET

        tags: ["Base Legal Policies With Namespace"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        offset: (offset) OPTIONAL int in query

        limit: (limit) REQUIRED int in query

    Responses:
        200: OK - List[RetrievePolicyTypeResponse] (successful operation)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = RetrieveAllPolicyTypes1.create(
        limit=limit,
        offset=offset,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(RetrievePolicyCountry1)
def retrieve_policy_country_1(
    base_policy_id: str,
    country_code: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Retrieve a Base Legal Policy based on a Particular Country (retrievePolicyCountry_1)

    Retrieve a Base Legal Policy based on a Particular Country.

    Properties:
        url: /agreement/admin/namespaces/{namespace}/base-policies/{basePolicyId}/countries/{countryCode}

        method: GET

        tags: ["Base Legal Policies With Namespace"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        base_policy_id: (basePolicyId) REQUIRED str in path

        country_code: (countryCode) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - RetrievePolicyResponse (successful operation)

        404: Not Found - ErrorEntity (40031: errors.net.accelbyte.platform.legal.base_policy_not_found)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = RetrievePolicyCountry1.create(
        base_policy_id=base_policy_id,
        country_code=country_code,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(RetrievePolicyCountry1)
async def retrieve_policy_country_1_async(
    base_policy_id: str,
    country_code: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Retrieve a Base Legal Policy based on a Particular Country (retrievePolicyCountry_1)

    Retrieve a Base Legal Policy based on a Particular Country.

    Properties:
        url: /agreement/admin/namespaces/{namespace}/base-policies/{basePolicyId}/countries/{countryCode}

        method: GET

        tags: ["Base Legal Policies With Namespace"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        base_policy_id: (basePolicyId) REQUIRED str in path

        country_code: (countryCode) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - RetrievePolicyResponse (successful operation)

        404: Not Found - ErrorEntity (40031: errors.net.accelbyte.platform.legal.base_policy_not_found)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = RetrievePolicyCountry1.create(
        base_policy_id=base_policy_id,
        country_code=country_code,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(RetrieveSinglePolicy1)
def retrieve_single_policy_1(
    base_policy_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Retrieve a Base Legal Policy (retrieveSinglePolicy_1)

    Retrieve a base policy.

    Properties:
        url: /agreement/admin/namespaces/{namespace}/base-policies/{basePolicyId}

        method: GET

        tags: ["Base Legal Policies With Namespace"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        base_policy_id: (basePolicyId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - RetrieveBasePolicyResponse (successful operation)

        404: Not Found - ErrorEntity (40041: errors.net.accelbyte.platform.legal.policy_not_found)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = RetrieveSinglePolicy1.create(
        base_policy_id=base_policy_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(RetrieveSinglePolicy1)
async def retrieve_single_policy_1_async(
    base_policy_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Retrieve a Base Legal Policy (retrieveSinglePolicy_1)

    Retrieve a base policy.

    Properties:
        url: /agreement/admin/namespaces/{namespace}/base-policies/{basePolicyId}

        method: GET

        tags: ["Base Legal Policies With Namespace"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        base_policy_id: (basePolicyId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - RetrieveBasePolicyResponse (successful operation)

        404: Not Found - ErrorEntity (40041: errors.net.accelbyte.platform.legal.policy_not_found)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = RetrieveSinglePolicy1.create(
        base_policy_id=base_policy_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

"""Auto-generated package that contains models used by the AccelByte Gaming Services Session Service."""

__version__ = "3.20.0"
__author__ = "AccelByte"
__email__ = "dev@accelbyte.net"

# pylint: disable=line-too-long

# certificate
from .wrappers import admin_upload_x_box_certificate
from .wrappers import admin_upload_x_box_certificate_async

# config
from .wrappers import admin_get_log_config
from .wrappers import admin_get_log_config_async
from .wrappers import admin_patch_update_log_config
from .wrappers import admin_patch_update_log_config_async

# configuration_template
from .wrappers import admin_create_configuration_alert_v1
from .wrappers import admin_create_configuration_alert_v1_async
from .wrappers import admin_create_configuration_template_v1
from .wrappers import admin_create_configuration_template_v1_async
from .wrappers import admin_delete_configuration_alert_v1
from .wrappers import admin_delete_configuration_alert_v1_async
from .wrappers import admin_delete_configuration_template_v1
from .wrappers import admin_delete_configuration_template_v1_async
from .wrappers import admin_get_all_configuration_templates_v1
from .wrappers import admin_get_all_configuration_templates_v1_async
from .wrappers import admin_get_configuration_alert_v1
from .wrappers import admin_get_configuration_alert_v1_async
from .wrappers import admin_get_configuration_template_v1
from .wrappers import admin_get_configuration_template_v1_async
from .wrappers import admin_get_dsmc_configuration
from .wrappers import admin_get_dsmc_configuration_async
from .wrappers import admin_sync_dsmc_configuration
from .wrappers import admin_sync_dsmc_configuration_async
from .wrappers import admin_update_configuration_alert_v1
from .wrappers import admin_update_configuration_alert_v1_async
from .wrappers import admin_update_configuration_template_v1
from .wrappers import admin_update_configuration_template_v1_async

# dsmc_default_configuration
from .wrappers import admin_get_dsmc_configuration_default
from .wrappers import admin_get_dsmc_configuration_default_async

# environment_variable
from .wrappers import admin_list_environment_variables
from .wrappers import admin_list_environment_variables_async

# game_session
from .wrappers import admin_delete_bulk_game_sessions
from .wrappers import admin_delete_bulk_game_sessions_async
from .wrappers import admin_kick_game_session_member
from .wrappers import admin_kick_game_session_member_async
from .wrappers import admin_query_game_sessions
from .wrappers import admin_query_game_sessions_async
from .wrappers import admin_query_game_sessions_by_attributes
from .wrappers import admin_query_game_sessions_by_attributes_async
from .wrappers import admin_set_ds_ready
from .wrappers import admin_set_ds_ready_async
from .wrappers import admin_update_game_session_member
from .wrappers import admin_update_game_session_member_async
from .wrappers import append_team_game_session
from .wrappers import append_team_game_session_async
from .wrappers import create_game_session
from .wrappers import create_game_session_async
from .wrappers import delete_game_session
from .wrappers import delete_game_session_async
from .wrappers import game_session_generate_code
from .wrappers import game_session_generate_code_async
from .wrappers import get_game_session
from .wrappers import get_game_session_async
from .wrappers import get_game_session_by_pod_name
from .wrappers import get_game_session_by_pod_name_async
from .wrappers import get_session_server_secret
from .wrappers import get_session_server_secret_async
from .wrappers import join_game_session
from .wrappers import join_game_session_async
from .wrappers import leave_game_session
from .wrappers import leave_game_session_async
from .wrappers import patch_update_game_session
from .wrappers import patch_update_game_session_async
from .wrappers import public_game_session_cancel
from .wrappers import public_game_session_cancel_async
from .wrappers import public_game_session_invite
from .wrappers import public_game_session_invite_async
from .wrappers import public_game_session_reject
from .wrappers import public_game_session_reject_async
from .wrappers import public_kick_game_session_member
from .wrappers import public_kick_game_session_member_async
from .wrappers import public_promote_game_session_leader
from .wrappers import public_promote_game_session_leader_async
from .wrappers import public_query_game_sessions_by_attributes
from .wrappers import public_query_game_sessions_by_attributes_async
from .wrappers import public_query_my_game_sessions
from .wrappers import public_query_my_game_sessions_async
from .wrappers import public_revoke_game_session_code
from .wrappers import public_revoke_game_session_code_async
from .wrappers import public_session_join_code
from .wrappers import public_session_join_code_async
from .wrappers import update_game_session
from .wrappers import update_game_session_async
from .wrappers import update_game_session_backfill_ticket_id
from .wrappers import update_game_session_backfill_ticket_id_async

# global_configuration
from .wrappers import admin_delete_global_configuration
from .wrappers import admin_delete_global_configuration_async
from .wrappers import admin_list_global_configuration
from .wrappers import admin_list_global_configuration_async
from .wrappers import admin_update_global_configuration
from .wrappers import admin_update_global_configuration_async

# max_active
from .wrappers import admin_get_member_active_session
from .wrappers import admin_get_member_active_session_async
from .wrappers import admin_reconcile_max_active_session
from .wrappers import admin_reconcile_max_active_session_async

# native_session
from .wrappers import admin_get_list_native_session
from .wrappers import admin_get_list_native_session_async

# operations
from .wrappers import get_healthcheck_info
from .wrappers import get_healthcheck_info_async
from .wrappers import get_healthcheck_info_v1
from .wrappers import get_healthcheck_info_v1_async

# party
from .wrappers import admin_delete_bulk_parties
from .wrappers import admin_delete_bulk_parties_async
from .wrappers import admin_query_parties
from .wrappers import admin_query_parties_async
from .wrappers import admin_sync_native_session
from .wrappers import admin_sync_native_session_async
from .wrappers import public_create_party
from .wrappers import public_create_party_async
from .wrappers import public_generate_party_code
from .wrappers import public_generate_party_code_async
from .wrappers import public_get_party
from .wrappers import public_get_party_async
from .wrappers import public_party_cancel
from .wrappers import public_party_cancel_async
from .wrappers import public_party_invite
from .wrappers import public_party_invite_async
from .wrappers import public_party_join
from .wrappers import public_party_join_async
from .wrappers import public_party_join_code
from .wrappers import public_party_join_code_async
from .wrappers import public_party_kick
from .wrappers import public_party_kick_async
from .wrappers import public_party_leave
from .wrappers import public_party_leave_async
from .wrappers import public_party_reject
from .wrappers import public_party_reject_async
from .wrappers import public_patch_update_party
from .wrappers import public_patch_update_party_async
from .wrappers import public_promote_party_leader
from .wrappers import public_promote_party_leader_async
from .wrappers import public_query_my_parties
from .wrappers import public_query_my_parties_async
from .wrappers import public_revoke_party_code
from .wrappers import public_revoke_party_code_async
from .wrappers import public_update_party
from .wrappers import public_update_party_async

# platform_credential
from .wrappers import admin_delete_platform_credentials
from .wrappers import admin_delete_platform_credentials_async
from .wrappers import admin_delete_platform_credentials_by_platform_id
from .wrappers import admin_delete_platform_credentials_by_platform_id_async
from .wrappers import admin_get_platform_credentials
from .wrappers import admin_get_platform_credentials_async
from .wrappers import admin_sync_platform_credentials
from .wrappers import admin_sync_platform_credentials_async
from .wrappers import admin_update_platform_credentials
from .wrappers import admin_update_platform_credentials_async

# player
from .wrappers import admin_get_player_attributes
from .wrappers import admin_get_player_attributes_async
from .wrappers import admin_query_player_attributes
from .wrappers import admin_query_player_attributes_async
from .wrappers import public_delete_player_attributes
from .wrappers import public_delete_player_attributes_async
from .wrappers import public_get_bulk_player_current_platform
from .wrappers import public_get_bulk_player_current_platform_async
from .wrappers import public_get_player_attributes
from .wrappers import public_get_player_attributes_async
from .wrappers import public_store_player_attributes
from .wrappers import public_store_player_attributes_async

# recent_player
from .wrappers import admin_get_recent_player
from .wrappers import admin_get_recent_player_async
from .wrappers import admin_get_recent_team_player
from .wrappers import admin_get_recent_team_player_async
from .wrappers import public_get_recent_player
from .wrappers import public_get_recent_player_async
from .wrappers import public_get_recent_team_player
from .wrappers import public_get_recent_team_player_async

# session_storage
from .wrappers import admin_delete_user_session_storage
from .wrappers import admin_delete_user_session_storage_async
from .wrappers import admin_read_session_storage
from .wrappers import admin_read_session_storage_async
from .wrappers import admin_read_user_session_storage
from .wrappers import admin_read_user_session_storage_async
from .wrappers import public_update_insert_session_storage
from .wrappers import public_update_insert_session_storage_async
from .wrappers import public_update_insert_session_storage_leader
from .wrappers import public_update_insert_session_storage_leader_async



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.

from ._login import login
from ._login import login_client
from ._login import login_platform
from ._login import login_user
from ._login import logout
from ._login import refresh_login

from ._login import login_async
from ._login import login_client_async
from ._login import login_platform_async
from ._login import login_user_async
from ._login import logout_async
from ._login import refresh_login_async

from ._login import LoginTimerBase
from ._login import LoginClientTimer
from ._login import LoginPlatformTimer
from ._login import LoginUserTimer
from ._login import RefreshLoginTimer

from ._login import DEFAULT_AUTO_REFRESH
from ._login import DEFAULT_REFRESH_RATE

from ._login import set_on_demand_token_refresher
from ._login import OnDemandTokenRefresher

from ._jwt import parse_access_token

from ._utils import convert_bearer_auth_token_to_oauth_token_dict
